#include <stdio.h>
#include <stdlib.h>

#ifdef consistent_set
int CONSISTENT_ORDER = 1;
#else
int CONSISTENT_ORDER = 0;
#endif // consistent_set

#ifndef TREE_SUM
#define TREE_SUM tree_sum
#endif

typedef struct tree_node {
  long value;
  struct tree_node *left;
  struct tree_node *right;
  struct tree_node *next;  
} tree_node;

static tree_node *make_tree(int depth) {
  tree_node *tn;
  static tree_node *prev = NULL;

  tn = malloc(sizeof(tree_node));
  tn->value = depth;
  if (depth > 0) {
    if (CONSISTENT_ORDER) {
      tn->left = make_tree(depth - 1);
      tn->right = make_tree(depth - 1);
    } else {
      if (depth & 0x1) {
        tn->left = make_tree(depth - 1);
        tn->right = make_tree(depth - 1);
      } else {
        tn->right = make_tree(depth - 1);
        tn->left = make_tree(depth - 1);
      }
    }
  } else {
    tn->left = NULL;
    tn->right = NULL;
  }

  /* For list_sum: */
  tn->next = NULL;
  if (prev)
    prev->next = tn;
  prev = tn;

  return tn;
}

static long tree_sum(tree_node *tn) {
  if (tn == NULL)
    return 0;
  else {
    long sum = tn->value;
    sum += tree_sum(tn->left);
    sum += tree_sum(tn->right);
    return sum;
  }
}

static long backward_tree_sum(tree_node *tn) {
  if (tn == NULL)
    return 0;
  else {
    long sum = tn->value;
    sum += backward_tree_sum(tn->right);
    sum += backward_tree_sum(tn->left);
    return sum;
  }
}

#define BM_TREE_DEPTH 23
#define BM_ITERS      10

int main() {
  tree_node *tn;
  int i;
  long sum;
  if(CONSISTENT_ORDER)
    {
      printf("%s", "consistent_set is set!\n");
    }

  tn = make_tree(BM_TREE_DEPTH);

  for (i = 0; i < BM_ITERS; i++) {
    /* Pick one of the sum functions: */
    sum += TREE_SUM(tn);
  }

  printf("%ld\n", sum);

  return 0;
}
